/*
 * support/export/xtab.c
 *
 * Interface to the xtab file.
 *
 * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
 */

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include "xmalloc.h"
#include "nfslib.h"
#include "exportfs.h"
#include "xio.h"
#include "xlog.h"

static int
extab_read(char *xtab)
{
	struct exportent	*xp;
	nfs_export		*exp;
	int			lockid;

	if ((lockid = xflock(xtab, "r")) < 0)
		return 0;
	setexportent(xtab, "r");
	while ((xp = getexportent()) != NULL) {
		if (!(exp = export_lookup(xp->e_hostname, xp->e_path)) &&
		    !(exp = export_create(xp))) {
			continue;
		}
		exp->m_xtabent = 1;
		if (xp->e_flags & NFSEXP_NEGATIVE)
		  exp->m_mayexport = 0;
		else
		  exp->m_mayexport = 1;
	}
	endexportent();
	xfunlock(lockid);

	return 0;
}

int
etab_read ()
{
	return extab_read (_PATH_ETAB);
}

int
xtab_read ()
{
	return extab_read (_PATH_XTAB);
}

int
etab_write()
{
	struct exportent	xe;
	nfs_export		*exp;
	int			lockid, i;
	char			*xtab = _PATH_ETAB;
	char			*xtabtmp = _PATH_ETABTMP;

	if ((lockid = xflock(xtab, "w")) < 0) {
		xlog(L_ERROR, "can't lock %s for writing", xtab);
		return 0;
	}
	setexportent(xtabtmp, "w");

	for (i = 0; i < MCL_MAXTYPES; i++) {
		for (exp = exportlist[i]; exp; exp = exp->m_next) {
			if (!exp->m_mayexport)
				continue;

			/* write out the export entry using the FQDN */
			xe = exp->m_export;
			strncpy(xe.e_hostname,
				exp->m_client->m_hostname,
				sizeof (xe.e_hostname) - 1);
			xe.e_hostname[sizeof (xe.e_hostname) - 1] = '\0';
			putexportent(&xe);
		}
	}
	endexportent();

	rename(xtabtmp, xtab);
	xfunlock(lockid);

	return 1;
}
